@@ -25,6 +25,14 @@ impl<T> Mutex<T>{
             &mut *(self.0.lock().unwrap().deref_mut() as *mut T)
         }
     }
+
+    pub fn data_ptr(&self) -> *mut T {
+        // There is no way to get without lock in loom
+        unsafe{
+            use std::ops::DerefMut;
+            self.0.lock().unwrap().deref_mut() as *mut T
+        }
+    }
 }
 
 #[cfg(loom)]
@@ -39,7 +47,8 @@ pub(crate) use std::sync::atomic::{AtomicPtr, AtomicUsize, AtomicU64, Ordering};
 pub(crate) use std::sync::Arc;
 
 #[cfg(not(loom))]
-pub(crate) use parking_lot::{Mutex};
+//pub(crate) use parking_lot::{Mutex};
+pub(crate) type Mutex<T> = lock_api::Mutex<spin::mutex::Mutex<(), spin::relax::Yield>, T>;
 
 #[cfg(not(loom))]
 pub(crate) use spin::mutex::{SpinMutex};
